@@ -52,8 +52,10 @@
 
 struct GsdA11yKeyboardManagerPrivate
 {
+        guint             start_idle_id;
         int               xkbEventBase;
         GdkDeviceManager *device_manager;
+        guint             device_added_id;
         gboolean          stickykeys_shortcut_val;
         gboolean          slowkeys_shortcut_val;
         GtkWidget        *stickykeys_alert;
@@ -103,8 +105,8 @@ set_devicepresence_handler (GsdA11yKeyboardManager *manager)
                 return;
 
         manager->priv->device_manager = device_manager;
-        g_signal_connect (G_OBJECT (device_manager), "device-added",
-                          G_CALLBACK (device_added_cb), manager);
+        manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
+                                                           G_CALLBACK (device_added_cb), manager);
 }
 
 static gboolean
@@ -967,17 +969,18 @@ start_a11y_keyboard_idle_cb (GsdA11yKeyboardManager *manager)
  out:
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
-
 gboolean
 gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
                                  GError                **error)
 {
         gnome_settings_profile_start (NULL);
 
-        g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -1020,7 +1023,7 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
         g_debug ("Stopping a11y_keyboard manager");
 
         if (p->device_manager != NULL) {
-                g_object_unref (p->device_manager);
+                g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 p->device_manager = NULL;
         }
 
@@ -1186,6 +1189,9 @@ gsd_a11y_keyboard_manager_finalize (GObject *object)
 
         g_return_if_fail (a11y_keyboard_manager->priv != NULL);
 
+        if (a11y_keyboard_manager->priv->start_idle_id != 0)
+                g_source_remove (a11y_keyboard_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_a11y_keyboard_manager_parent_class)->finalize (object);
 }
 
