@@ -49,6 +49,7 @@
 
 #ifdef HAVE_PULSE
 #include <canberra-gtk.h>
+#include <pulse/pulseaudio.h>
 #include "gvc-mixer-control.h"
 #endif /* HAVE_PULSE */
 
@@ -79,7 +80,10 @@ static const gchar introspection_xml[] =
 #define HIGH_CONTRAST "HighContrast"
 
 #define VOLUME_STEP 6           /* percents for one volume button press */
-#define MAX_VOLUME 65536.0
+/* FIXME: Remove when PA 0.9.23 is used */
+#ifndef PA_VOLUME_UI_MAX
+#define PA_VOLUME_UI_MAX pa_sw_volume_from_dB(+11.0)
+#endif
 
 #define GSD_MEDIA_KEYS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_MEDIA_KEYS_MANAGER, GsdMediaKeysManagerPrivate))
 
@@ -115,6 +119,8 @@ struct GsdMediaKeysManagerPrivate
         GDBusConnection *connection;
         GDBusProxy      *xrandr_proxy;
         GCancellable    *cancellable;
+
+        guint            start_idle_id;
 };
 
 static void     gsd_media_keys_manager_class_init  (GsdMediaKeysManagerClass *klass);
@@ -621,10 +627,11 @@ do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 static void
 update_dialog (GsdMediaKeysManager *manager,
                guint vol,
+               guint max_vol,
                gboolean muted,
                gboolean sound_changed)
 {
-        vol = (int) (100 * (double) vol / PA_VOLUME_NORM);
+        vol = (int) (100 * (double) vol / (double) max_vol);
         vol = CLAMP (vol, 0, 100);
 
         dialog_init (manager);
@@ -647,80 +654,54 @@ static void
 do_sound_action (GsdMediaKeysManager *manager,
                  int                  type)
 {
-        gboolean muted;
-        guint vol, norm_vol_step;
-        int vol_step;
+        gboolean old_muted, new_muted;
+        guint old_vol, new_vol, max_vol, norm_vol_step;
         gboolean sound_changed;
 
         if (manager->priv->stream == NULL)
                 return;
 
-        vol_step = VOLUME_STEP;
-
-        norm_vol_step = PA_VOLUME_NORM * vol_step / 100;
+        max_vol = gvc_mixer_stream_get_can_decibel (manager->priv->stream) ? PA_VOLUME_UI_MAX : PA_VOLUME_NORM;
+        norm_vol_step = max_vol * VOLUME_STEP / 100;
 
         /* FIXME: this is racy */
-        vol = gvc_mixer_stream_get_volume (manager->priv->stream);
-        muted = gvc_mixer_stream_get_is_muted (manager->priv->stream);
+        new_vol = old_vol = gvc_mixer_stream_get_volume (manager->priv->stream);
+        new_muted = old_muted = gvc_mixer_stream_get_is_muted (manager->priv->stream);
         sound_changed = FALSE;
 
         switch (type) {
         case MUTE_KEY:
-                muted = !muted;
-                gvc_mixer_stream_change_is_muted (manager->priv->stream, muted);
-                sound_changed = TRUE;
+                new_muted = !old_muted;
                 break;
         case VOLUME_DOWN_KEY:
-                if (!muted && (vol <= norm_vol_step)) {
-                        muted = !muted;
-                        vol = 0;
-                        gvc_mixer_stream_change_is_muted (manager->priv->stream, muted);
-                        if (gvc_mixer_stream_set_volume (manager->priv->stream, vol) != FALSE) {
-                                gvc_mixer_stream_push_volume (manager->priv->stream);
-                                sound_changed = TRUE;
-                        }
+                if (old_vol <= norm_vol_step) {
+                        new_vol = 0;
+                        new_muted = TRUE;
                 } else {
-                        if (vol <= norm_vol_step)
-                                vol = 0;
-                        else
-                                vol = vol - norm_vol_step;
-                        if (gvc_mixer_stream_set_volume (manager->priv->stream, vol) != FALSE) {
-                                gvc_mixer_stream_push_volume (manager->priv->stream);
-                                sound_changed = TRUE;
-                        }
+                        new_vol = old_vol - norm_vol_step;
                 }
                 break;
         case VOLUME_UP_KEY:
-                if (muted) {
-                        muted = !muted;
-                        if (vol == 0) {
-                               vol = vol + norm_vol_step;
-                               gvc_mixer_stream_change_is_muted (manager->priv->stream, muted);
-                               if (gvc_mixer_stream_set_volume (manager->priv->stream, vol) != FALSE) {
-                                        gvc_mixer_stream_push_volume (manager->priv->stream);
-                                        sound_changed = TRUE;
-                               }
-                        } else {
-                                gvc_mixer_stream_change_is_muted (manager->priv->stream, muted);
-                                sound_changed = TRUE;
-                        }
-                } else {
-                        if (vol < MAX_VOLUME) {
-                                if (vol + norm_vol_step >= MAX_VOLUME) {
-                                        vol = MAX_VOLUME;
-                                } else {
-                                        vol = vol + norm_vol_step;
-                                }
-                                if (gvc_mixer_stream_set_volume (manager->priv->stream, vol) != FALSE) {
-                                        gvc_mixer_stream_push_volume (manager->priv->stream);
-                                        sound_changed = TRUE;
-                                }
-                        }
-                }
+                new_muted = FALSE;
+                /* When coming out of mute only increase the volume if it was 0 */
+                if (!old_muted || old_vol == 0)
+                        new_vol = MIN (old_vol + norm_vol_step, max_vol);
                 break;
         }
 
-        update_dialog (manager, vol, muted, sound_changed);
+        if (old_muted != new_muted) {
+                gvc_mixer_stream_change_is_muted (manager->priv->stream, new_muted);
+                sound_changed = TRUE;
+        }
+
+        if (old_vol != new_vol) {
+                if (gvc_mixer_stream_set_volume (manager->priv->stream, new_vol) != FALSE) {
+                        gvc_mixer_stream_push_volume (manager->priv->stream);
+                        sound_changed = TRUE;
+                }
+        }
+
+        update_dialog (manager, new_vol, max_vol, new_muted, sound_changed);
 }
 
 static void
@@ -1419,6 +1400,8 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
@@ -1452,7 +1435,7 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
 
         gnome_settings_profile_end ("gvc_mixer_control_new");
 #endif /* HAVE_PULSE */
-        g_idle_add ((GSourceFunc) start_media_keys_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_media_keys_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -1619,6 +1602,9 @@ gsd_media_keys_manager_finalize (GObject *object)
 
         g_return_if_fail (media_keys_manager->priv != NULL);
 
+        if (media_keys_manager->priv->start_idle_id != 0)
+                g_source_remove (media_keys_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_media_keys_manager_parent_class)->finalize (object);
 }
 
